from typing import List, Tuple


def pick_entity_from_bio_labels(
        pairs: List[Tuple[str, str]],
        with_offset: bool = False
) -> List[Tuple[str, str]]:
    """
    Parameters
    ----------
    pairs: List of tuple pairs, each pair contains a token and a bio tag
    with_offset: whether to return locations for the entities

    Returns
    -------
    List of entity pairs, each pair contains an entity and entity type (migtht also a start and end index)
    """

    def collect(span: List[Tuple[str, str]]):
        res = []
        for c, t in span:
            if t.endswith("O"):
                break
            res.append(c)
        return "".join(res), span[0][1].split("-")[-1]

    without_lasto = False
    if pairs and pairs[-1][1] != "O":
        without_lasto = True
        pairs.append(("#", "O"))
    bidx = []
    for i, (c, t) in enumerate(pairs):
        if t.startswith("B-"):
            bidx.append(i)
    bidx.append(len(pairs) - 1)
    res = []
    for i in range(len(bidx) - 1):
        start, end = bidx[i], bidx[i + 1]
        span = pairs[start: end]
        word, tag = collect(span)
        if with_offset:
            tup = (word, tag, start, start + len(word))
        else:
            tup = (word, tag)
        res.append(tup)
    if without_lasto:
        pairs.pop()
    return res



from pnlp.piop import read_file, read_lines, read_json, read_yaml, read_csv, read_pickle
from pnlp.piop import write_file, write_json, write_pickle, check_dir
from pnlp.pcut import cut_sentence, cut_zhchar, cut_part, combine_bucket
from pnlp.pcut import psent, psubsent

from pnlp.piop import Reader, Dict
from pnlp.ptxt import Regex, Text, Length
from pnlp.pnorm import NumNorm
from pnlp.penh import TokenLevelSampler, SentenceLevelSampler
from pnlp.ptrans import pick_entity_from_bio_labels
from pnlp.pmag import MagicDict
from pnlp.stopwords import StopWords
from pnlp.stopwords import chinese_stopwords, english_stopwords

from pnlp.utils import pstr, concurring, divide2int
from pnlp.utils import generate_batches_by_num, generate_batches_by_size


num_norm = NumNorm()
reg = Regex()
reader = Reader()
tlsampler = TokenLevelSampler()
slsampler = SentenceLevelSampler()


__title__ = "pnlp"
__version__ = "0.4.6"
__author__ = "Yam"
__license__ = "Apache-2.0"
__copyright__ = "Copyright 2019, 2020 Yam"
__all__ = ["Reader",
           "Text", "Regex", "Length",
           "MagicDict",
           "NumNorm",
           "StopWords",
           "TokenLevelSampler", "SentenceLevelSampler"
           ]
